var fs = require('fs');
var child_process = require('child_process');
var qewd = require('../../lib/master');

function setup(isDocker) {

  var cwd = process.cwd();

  if (isDocker) {
    cwd = cwd + '/mapped';
    var uuid = require('uuid/v4'); // loaded as dependency of ewd-session
  }

  console.log('isDocker: ' + isDocker);
  console.log('cwd = ' + cwd);

  var config_data;
  try {
    config_data = require(cwd + '/configuration/config.json');
  }
  catch(err) {
    config_data = {
      conductor: {}
    };
  }

  if (!fs.existsSync(cwd + '/www')) {
    fs.mkdirSync(cwd + '/www');
  }

  if (config_data.conductor) {
    var cmd;
    if (config_data.conductor['qewd-monitor'] !== false) {
      console.log('enabling qewd-monitor');
      cmd = 'ln -sf ' + cwd + '/node_modules/qewd-monitor/www ' + cwd + '/www/qewd-monitor';
      child_process.execSync(cmd, {stdio:[0,1,2]});
    }
    else {
      if (fs.existsSync(cwd + '/www/qewd-monitor')) {
        cmd = 'unlink ' + cwd + '/www/qewd-monitor';
        child_process.execSync(cmd, {stdio:[0,1,2]});
      }
    }
  }

  var routes_data;
  try {
    routes_data = require(cwd + '/configuration/routes.json');
  }
  catch(err) {
    routes_data = [];
  }

  var transform = require('qewd-transform-json').transform;
  var helpers = {};

  var config_template = {
    managementPassword: '=> either(conductor.qewd.managementPassword, "keepThisSecret!")',
    serverName: '=> either(conductor.qewd.serverName, "QEWD Server")',
    port: '=> either(conductor.qewd.port, 8080)',
    poolSize: '=> either(conductor.qewd.poolSize, 2)',
    database: {
      type: 'gtm'
    },
  };

  var routes = [];
  var roots = {};
  var config = transform(config_template, config_data, helpers);

  if (isDocker) {

    // Add in microservice definitions if present

    console.log('config_data.microservices: ' + JSON.stringify(config_data.microservices, null, 2));

    if (config_data.microservices && Array.isArray(config_data.microservices)) {
      config.u_services = {
        destinations: {},
        routes: []
      };
      var destinations = config.u_services.destinations;
      config_data.microservices.forEach(function(microservice) {  
        destinations[microservice.name] = {
          host: microservice.host + ':' + microservice.port,
          application: microservice.name
        };
      });

      routes_data.forEach(function(route) {
        if (route.on_microservice) {
          config.u_services.routes.push({
            path: route.uri,
            method: route.method,
            destination: route.on_microservice
          });
        }
      });

      if (!config_data.jwt || !config_data.jwt.secret) {
        config_data.jwt = {
          secret: uuid()
        };
      }
      config.jwt = config_data.jwt;
    }
  }

  routes_data.forEach(function(route) {
    if (route.else) return;
    var path_root = '/' + route.uri.split('/')[1];
    if (!roots[path_root]) {

      var routeObj = {
        path: path_root,
        module: __dirname + '/handlers'
      };

      var beforeRouterPath = cwd + '/handlers' + path_root + '/beforeRouter.js';
      console.log('beforeRouterPath: ' + beforeRouterPath);
      if (fs.existsSync(beforeRouterPath)) {
        routeObj.beforeRouter = [require(beforeRouterPath)];
      }

      var afterRouterPath = cwd + '/handlers' + path_root + '/afterRouter.js';
      console.log('afterRouterPath: ' + afterRouterPath);
      if (fs.existsSync(afterRouterPath)) {
        routeObj.afterRouter = [require(afterRouterPath)];
      }

      routes.push(routeObj);
      roots[path_root] = true;
    }
  });

  return {
    routes: routes,
    config: config,
    cwd: cwd
  };
}

module.exports = function(isDocker) {

  var results = setup(isDocker);
  var config = results.config;
  var routes = results.routes;
  var cwd = results.cwd;

  console.log('master config: ' + JSON.stringify(config, null ,2));
  console.log('routes: ' + JSON.stringify(routes, null, 2));

  var onStartedPath = cwd + '/conductor/onStarted.js';
  var onStartedPath2 = cwd + '/onStarted.js';

  if (isDocker) {

    var exportObj = {
      config: config,
      routes: routes
    };

    if (fs.existsSync(onStartedPath)) {
      exportObj.onStarted = require(onStartedPath);
    }
    else if (fs.existsSync(onStartedPath2)) {
      exportObj.onStarted = require(onStartedPath2);
    }
    return exportObj;
  }
  else {

    try {
      console.log('Running down YottaDB...');
      child_process.execSync(process.env.ydb_dist + '/mupip rundown -region DEFAULT', {stdio:[0,1,2]});
      child_process.execSync(process.env.ydb_dist + '/mupip rundown -region qewdreg', {stdio:[0,1,2]});
      console.log('Rundown completed');
    }
    catch(err) {
      console.log('Error running down YottaDB: ' + err);
      console.log('Recovering journal...');
      child_process.execSync(process.env.ydb_dist + '/mupip journal -recover -backward ' + process.env.ydb_dir + '/' + process.env.ydb_rel + '/g/yottadb.mjl', {stdio:[0,1,2]});
      console.log('Journal recovered');
    }

    var q = qewd.start(config, routes);

    if (fs.existsSync(onStartedPath)) {
      require(onStartedPath).call(q, config);
    }
    else if (fs.existsSync(onStartedPath2)) {
      require(onStartedPath2).call(q, config);
    }
  }
};
